#include <OneWire.h>
#include <IRremote.h>
#include <IRremoteInt.h>

/*
 * Licence : CC -by -sa
 * Auteur : Matthieu Cargnelli
 */

#define FAN_SPEED_LOWEST 0b1100  //3
#define FAN_SPEED_LOW 0b0010	//4
#define FAN_SPEED_MEDIUM 0b1010  //5
#define FAN_SPEED_HIGH 0b0110    //6
#define FAN_SPEED_HIGHEST 0b1110 //7
#define FAN_SPEED_AUTO 0b0101 	//10

#define SWING_LOWEST 0b10000000  //1
#define SWING_LOW 0b01000000	 //2
#define SWING_MEDIUM 0b11000000  //3
#define SWING_HIGH 0b00100000    //4
#define SWING_HIGHEST 0b10100000 //5
#define SWING_AUTO 0b11110000	 //15

#define MODE_COOL 0b1100
#define MODE_HEAT 0b0010
#define MODE_DRY 0b0100
#define MODE_AUTO 0b0000

#define ON 0b10000000
#define OFF 0b00000000

#define OPTION_POWERFUL 0b10000000
#define OPTION_QUIET 0b00000100
#define OPTION_NONE 0b00000000

// index of data in IR payload, also used as identifier of the next data expected
#define ON_OFF_MODE 5
#define TEMPERATURE 6
#define FLUX 8
#define OPTION 13
#define CHECKSUM 18

// TEMPERATURE VALUE SIZE
#define TEMP_SIZE 6

// COMMANDS CODES
#define SEND_IRCOMMAND 2
#define SEND_STATUS 3
#define SEND_OFF_COMMAND 4
#define RADIO_MESSAGE 5

// MAXIMUM SIZE OF THE PAYLOAD
#define MAX_PAYLOAD 20

#define PAYLOAD_SIZE 19

#define INTRO_SIZE 8

// payload description -- IR commands timings and info
const int freq = 38; // 38 KHz
const int irOne = 1300;
const int irZero = 400;
const int irSpace = 400;
const int irSeparator = 9800;
const int irLock1 = 3500;
const int irLock2 = 1700;
const byte irIntro[INTRO_SIZE]={
  0x40,0x04,0x07,0x20,
  0x00,0x00,0x00,0x60}; // hexadecimal code for opening sequence

byte payload[PAYLOAD_SIZE];

boolean expectData = false;

byte command = 0;
boolean commandComplete = false;
long timeout;
unsigned char message[20];

long usbTimeoutMs = 1000;

// IR sender (from IRremote.h) - sets the pin and PWM properly
IRsend irsend;

const int checkLEDPin = 2;
int inputIndex = 0;
void setup()
{	
  Serial.begin(9600);
  pinMode(checkLEDPin,OUTPUT);
  flashLED(10);
}

void flashLED(){
  flashLED(1);
}

void flashLED(int times){
  for (int i = 0 ; i < times ; i++){
    digitalWrite(checkLEDPin, HIGH);
    delay (30);
    digitalWrite(checkLEDPin, LOW);
    if (i < times - 1){
      delay(30);
    }
  }
}
void loop()
{
  // when a command has been received from USB, execute it
  if (commandComplete) 
  {
    if (command == SEND_IRCOMMAND){
      emitIRCommand(payload, PAYLOAD_SIZE);
      Serial.println("OK"); // response to the caller
      inputIndex = 0;
    } 
    command = 0;
    commandComplete = false;
    flashLED(2);
  }

}

void emitIRCommand(byte* payload, int payloadSize){
  int l = 0;
  // set the PWM for the LED to 38KHz
  irsend.enableIROut(38);
  // send the Lock sequence
  irsend.mark(irLock1);
  irsend.space(irLock2);

  // send the intro part
  for (int s = 0 ; s < INTRO_SIZE ; s++ ){
    for (int i = 7 ; i >= 0 ; i--){ // Most Significant Bit comes first
      irsend.mark(irSpace);
      irsend.space((irIntro[s] >> i) &  1 == 1 ? irOne : irZero);
    }
  }
  // send the separators and Lock sequence
  irsend.mark(irSpace);
  irsend.space(irSeparator);
  irsend.mark(irLock1);
  irsend.space(irLock2);

  // send the payload itself
  for (int s = 0 ; s < payloadSize ; s++ ){
    for (int i = 7 ; i >= 0 ; i--){ // Most Significant Bit comes first
      irsend.mark(irSpace);
      irsend.space((payload[s] >> i) & 1 == 1 ? irOne : irZero);
    }
  }
  // end communication
  irsend.mark(irSpace);
  irsend.space(0); 
}

void serialEvent() {
  // this method is called whenever there is data coming from the serial link
  boolean timeoutOK;
  while (Serial.available()) {
    if (!expectData){
      // get the new byte:
      byte in = Serial.read();
      char inChar = (char)in; 
      switch(inChar){
      case 'I':
        //incomming IR message to send - the payload will follow
        commandComplete = false;
        command = SEND_IRCOMMAND;
        expectData = true;
        break;
      default:
        Serial.println("Unexpected parameter");
        return;
      }
    }else{ // data is expected - the order was received (SEND_IRCOMMAND), the payload is expected
      timeout = millis()+usbTimeoutMs;
      timeoutOK = true;
      // wait for the payload to be available. The payload is sent as characters representing hex value, 
      // so we'll have 2 chars per byte (from 00 to FF), hence the PAYLOAD_SIZE*2 value
      while (Serial.available() < PAYLOAD_SIZE*2 && timeoutOK){
        timeoutOK = (timeout > millis());
      }
      if (timeoutOK){
        if (command == SEND_IRCOMMAND){
          // hexConvert[] will be used to convert 2 chars from 00 to FF to an HEXA value through sscanf
          char hexConvert[5] = "0x";
          // ASCII is received
          for (int i = 0 ; i < 19 ; i++){
            hexConvert[2] = Serial.read();
            hexConvert[3] = Serial.read();
            hexConvert[4] = '\0';
            // convert the received chars to a numeric value in payload[i] 
            sscanf(hexConvert, "%x", &(payload[i]));
          }
          // in the end, the command is complete
          commandComplete = true;        
        }
      }
      else{
        Serial.println("Error, timeout while waiting for data");
        command = 0;
        expectData = false;
        return;
      }
      expectData = false;
    }
  }
}
